@@ -7,40 +7,17 @@
 # See https://aboutcode.org for more information about nexB OSS projects.
 #
 
-import logging
-import sys
 import warnings
 
-import attr
 import saneyaml
-from commoncode import filetype
 from packageurl import PackageURL
 
 from packagedcode import models
 from packagedcode.utils import combine_expressions
-
-TRACE = False
-
-
-def logger_debug(*args):
-    pass
-
-
-logger = logging.getLogger(__name__)
-
-if TRACE:
-    logging.basicConfig(stream=sys.stdout)
-    logger.setLevel(logging.DEBUG)
-
-    def logger_debug(*args):
-        return logger.debug(''.join(isinstance(a, str) and a or repr(a) for a in args))
-
 """
 Collect data from Dart pub packages.
 See https://dart.dev/tools/pub/pubspec
-"""
 
-"""
 TODO:
 - license is only in a LICENSE file
   https://dart.dev/tools/pub/publishing#preparing-to-publish
@@ -56,43 +33,52 @@ is limited and only returns all versions of a package
 See https://github.com/dart-lang/pub/blob/master/doc/repository-spec-v2.md
 """
 
+# FIXME: warnings reported here DO NOT work. We should have a better way
 
-@attr.s()
-class PubspecPackage(models.Package):
-    metafiles = ('pubspec.yaml', 'pubspec.lock',)
-    extensions = ('.yaml', '.lock',)
-    default_type = 'pubspec'
-    default_primary_language = 'dart'
-    default_web_baseurl = 'https://pub.dev/packages'
-    default_download_baseurl = 'https://pub.dartlang.org/packages'
-    default_api_baseurl = 'https://pub.dev/api/packages'
+
+class BaseDartPubspecHandler(models.DatafileHandler):
 
     @classmethod
-    def recognize(cls, location):
-        if is_pubspec_yaml(location):
-            yield parse_pub(location)
-        elif is_pubspec_lock(location):
-            yield parse_lock(location)
+    def assemble(cls, package_data, resource, codebase, package_adder):
+        datafile_name_patterns = \
+            DartPubspecYamlHandler.path_patterns + DartPubspecLockHandler.path_patterns
 
-    def repository_homepage_url(self, baseurl=default_web_baseurl):
-        return f'{baseurl}/{self.name}/versions/{self.version}'
+        if resource.has_parent():
+            dir_resource=resource.parent(codebase)
+        else:
+            dir_resource=resource
 
-    def repository_download_url(self, baseurl=default_download_baseurl):
-        # A URL should be in the form of:
-        # https://pub.dartlang.org/packages/url_launcher/versions/6.0.9.tar.gz
-        # And it may resolve to:
-        # https://storage.googleapis.com/pub-packages/packages/http-0.13.2.tar.gz
-        # as seen in the pub.dev web pages
-        return f'{baseurl}/{self.name}/versions/{self.version}.tar.gz'
+        yield from cls.assemble_from_many_datafiles(
+            datafile_name_patterns=datafile_name_patterns,
+            directory=dir_resource,
+            codebase=codebase,
+            package_adder=package_adder,
+        )
 
-    def api_data_url(self, baseurl=default_api_baseurl):
-        return f'{baseurl}/{self.name}/versions/{self.version}'
-
-    def compute_normalized_license(self):
-        return compute_normalized_license(self.declared_license)
+    @classmethod
+    def compute_normalized_license(cls, package):
+        return compute_normalized_license(package.declared_license)
 
 
-def compute_normalized_license(declared_license, location=None):
+class DartPubspecYamlHandler(BaseDartPubspecHandler):
+    datasource_id = 'pubspec_yaml'
+    path_patterns = ('*pubspec.yaml',)
+    default_package_type = 'pubspec'
+    default_primary_language = 'dart'
+    description = 'Dart pubspec manifest'
+    documentation_url = 'https://dart.dev/tools/pub/pubspec'
+
+    @classmethod
+    def parse(cls, location):
+        with open(location) as inp:
+            pubspec_data = saneyaml.load(inp.read())
+
+        package_data = build_package(pubspec_data)
+        if package_data:
+            yield package_data
+
+
+def compute_normalized_license(declared_license):
     """
     Return a normalized license expression string detected from a list of
     declared license items.
@@ -115,49 +101,27 @@ def compute_normalized_license(declared_license, location=None):
         return combine_expressions(detected_licenses)
 
 
-def parse_pub(location, compute_normalized_license=False):
-    """
-    Return a PubspecPackage constructed from the pubspec.yaml file at ``location``
-    or None.
-    """
-    if not is_pubspec_yaml(location):
-        return
-    with open(location) as inp:
-        package_data = saneyaml.load(inp.read())
+class DartPubspecLockHandler(BaseDartPubspecHandler):
+    datasource_id = 'pubspec_lock'
+    path_patterns = ('*pubspec.lock',)
+    default_package_type = 'pubspec'
+    default_primary_language = 'dart'
+    description = 'Dart pubspec lockfile'
+    documentation_url = 'https://web.archive.org/web/20220330081004/https://gpalma.pt/blog/what-is-the-pubspec-lock/'
 
-    package = build_package(package_data)
-    if package and compute_normalized_license:
-        package.compute_normalized_license()
-    return package
+    @classmethod
+    def parse(cls, location):
+        with open(location) as inp:
+            locks_data = saneyaml.load(inp.read())
 
+        dependencies = list(collect_locks(locks_data))
 
-def file_endswith(location, endswith):
-    """
-    Check if the file at ``location`` ends with ``endswith`` string or tuple.
-    """
-    return filetype.is_file(location) and location.endswith(endswith)
-
-
-def is_pubspec_yaml(location):
-    return file_endswith(location, 'pubspec.yaml')
-
-
-def is_pubspec_lock(location):
-    return file_endswith(location, 'pubspec.lock')
-
-
-def parse_lock(location):
-    """
-    Yield PubspecPackages dependencies constructed from the pubspec.lock file at
-    ``location``.
-    """
-    if not is_pubspec_lock(location):
-        return
-
-    with open(location) as inp:
-        locks_data = saneyaml.load(inp.read())
-
-    return PubspecPackage(dependencies=list(collect_locks(locks_data)))
+        yield models.PackageData(
+            datasource_id=cls.datasource_id,
+            type=cls.default_package_type,
+            primary_language=cls.default_primary_language,
+            dependencies=dependencies
+        )
 
 
 def collect_locks(locks_data):
@@ -282,15 +246,22 @@ def build_dep(name, version, scope, is_runtime=True, is_optional=False):
 
     if version.replace('.', '').isdigit():
         # version is pinned exactly if it is only made of dots and digits
-        purl = PackageURL(type='pubspec', name=name, version=version)
+        purl = PackageURL(
+            type='pubspec',
+            name=name,
+            version=version,
+        )
         is_resolved = True
     else:
-        purl = PackageURL(type='pubspec', name=name)
+        purl = PackageURL(
+            type='pubspec',
+            name=name,
+        )
         is_resolved = False
 
     dep = models.DependentPackage(
         purl=purl.to_string(),
-        requirement=version,
+        extracted_requirement=version,
         scope=scope,
         is_runtime=is_runtime,
         is_optional=is_optional,
@@ -311,6 +282,18 @@ def build_package(pubspec_data):
     vcs_url = pubspec_data.get('repository')
     download_url = pubspec_data.get('archive_url')
 
+    api_data_url = name and version and f'https://pub.dev/api/packages/{name}/versions/{version}'
+    repository_homepage_url = name and version and f'https://pub.dev/packages/{name}/versions/{version}'
+
+    # A URL should be in the form of:
+    # https://pub.dartlang.org/packages/url_launcher/versions/6.0.9.tar.gz
+    # And it may resolve to:
+    # https://storage.googleapis.com/pub-packages/packages/http-0.13.2.tar.gz
+    # as seen in the pub.dev web pages
+    repository_download_url = name and version and f'https://pub.dartlang.org/packages/{name}/versions/{version}.tar.gz'
+
+    download_url = download_url or repository_download_url
+
     # Author and authors are deprecated
     authors = []
     author = pubspec_data.get('author')
@@ -364,9 +347,13 @@ def build_package(pubspec_data):
     add_to_extra_if_present('executables')
     add_to_extra_if_present('publish_to')
 
-    package = PubspecPackage(
+    package = models.PackageData(
+        datasource_id=DartPubspecYamlHandler.datasource_id,
+        type=DartPubspecYamlHandler.default_primary_language,
+        primary_language=DartPubspecYamlHandler.default_primary_language,
         name=name,
         version=version,
+        download_url=download_url,
         vcs_url=vcs_url,
         description=description,
         declared_license=declared_license,
@@ -374,10 +361,12 @@ def build_package(pubspec_data):
         homepage_url=homepage_url,
         dependencies=package_dependencies,
         extra_data=extra_data,
+        repository_homepage_url=repository_homepage_url,
+        api_data_url=api_data_url,
+        repository_download_url=repository_download_url,
     )
 
-    if not download_url:
-        package.download_url = package.repository_download_url()
+    if not package.license_expression and package.declared_license:
+        package.license_expression = models.compute_normalized_license(package.declared_license)
 
     return package
-
