@@ -13,14 +13,18 @@ from scancode.pool import get_pool
 # Import early because of the side effects
 import scancode_config
 
+import logging
+import os
+import platform
+import sys
+import traceback
+
 from collections import defaultdict
 from functools import partial
-import os
-import logging
-import sys
 from time import sleep
 from time import time
-import traceback
+
+import commoncode
 
 # this exception is not available on posix
 try:
@@ -93,9 +97,9 @@ def print_examples(ctx, param, value):
 def print_version(ctx, param, value):
     if not value or ctx.resilient_parsing:
         return
-    click.echo('ScanCode version ' + scancode_config.__version__)
-    click.echo('ScanCode Output Format version ' + scancode_config.__output_format_version__)
-    click.echo('SPDX License list version ' + scancode_config.spdx_license_list_version)
+    click.echo(f'ScanCode version: {scancode_config.__version__}')
+    click.echo(f'ScanCode Output Format version: {scancode_config.__output_format_version__}')
+    click.echo(f'SPDX License list version: {scancode_config.spdx_license_list_version}')
     ctx.exit()
 
 
@@ -207,13 +211,13 @@ def validate_depth(ctx, param, value):
          f'[default: {DEFAULT_TIMEOUT} seconds]',
     help_group=cliutils.CORE_GROUP, sort_order=10, cls=PluggableCommandLineOption)
 
-@click.option('--quiet',
+@click.option('-q', '--quiet',
     is_flag=True,
     conflicting_options=['verbose'],
     help='Do not print summary or progress.',
     help_group=cliutils.CORE_GROUP, sort_order=20, cls=PluggableCommandLineOption)
 
-@click.option('--verbose',
+@click.option('-v', '--verbose',
     is_flag=True,
     conflicting_options=['quiet'],
     help='Print progress as file-by-file path instead of a progress bar. '
@@ -254,7 +258,7 @@ def validate_depth(ctx, param, value):
 @click.help_option('-h', '--help',
     help_group=cliutils.DOC_GROUP, sort_order=10, cls=PluggableCommandLineOption)
 
-@click.option('--about',
+@click.option('-A', '--about',
     is_flag=True,
     is_eager=True,
     expose_value=False,
@@ -262,7 +266,7 @@ def validate_depth(ctx, param, value):
     help='Show information about ScanCode and licensing and exit.',
     help_group=cliutils.DOC_GROUP, sort_order=20, cls=PluggableCommandLineOption)
 
-@click.option('--version',
+@click.option('-V', '--version',
     is_flag=True,
     is_eager=True,
     expose_value=False,
@@ -439,13 +443,10 @@ def scancode(
         cliutils.validate_option_dependencies(ctx)
         pretty_params = get_pretty_params(ctx, generic_paths=test_mode)
 
-        # warn for outdated version and/or check for updates
-        from scancode.outdated import check_scancode_version_locally
-        outdated = check_scancode_version_locally()
-
-        if not outdated and check_version:
-            from scancode.outdated import check_scancode_version_remotely
-            outdated = check_scancode_version_remotely()
+        # Check for updates
+        if check_version:
+            from scancode.outdated import check_scancode_version
+            outdated = check_scancode_version()
 
         # run proper
         success, _results = run_scan(
@@ -473,6 +474,7 @@ def scancode(
             **kwargs
         )
 
+        # echo outdated message if newer version is available
         if not quiet and outdated:
             echo_stderr(outdated, fg='yellow')
 
@@ -502,7 +504,9 @@ def run_scan(
     echo_func=None,
     timing=False,
     keep_temp_files=False,
+    # TODO: Review return_results as it does not return Packages and Dependencies
     return_results=True,
+    return_codebase=False,
     test_mode=False,
     test_slow_mode=False,
     test_error_mode=False,
@@ -520,6 +524,12 @@ def run_scan(
     error. See scancode() for arguments details.
     """
 
+    assert not (return_results is True and return_codebase is True), 'Only one of return_results and return_codebase can be True'
+
+    if return_codebase and max_in_memory > 0:
+        # We're keeping temp files otherwise the codebase is gone
+        keep_temp_files = True
+
     plugins_option_defaults = {clio.name: clio.default for clio in plugin_options}
     requested_options = dict(plugins_option_defaults)
     requested_options.update(kwargs)
@@ -665,7 +675,7 @@ def run_scan(
                    'selected when using only scan data.')
             raise ScancodeCliUsageError(msg)
 
-        if not output_plugins and not return_results:
+        if not output_plugins and not (return_results or return_codebase):
             msg = ('ERROR: Missing output option(s): at least one output '
                    'option is required to save scan results.')
             raise ScancodeCliUsageError(msg)
@@ -849,6 +859,14 @@ def run_scan(
         cle.output_format_version = scancode_config.__output_format_version__
         cle.notice = notice
         cle.options = pretty_params or {}
+        # useful for debugging
+        cle.extra_data['system_environment'] = system_environment = {}
+        system_environment['operating_system'] = commoncode.system.current_os
+        system_environment['cpu_architecture'] = commoncode.system.current_arch
+        system_environment['platform'] = platform.platform()
+        system_environment['platform_version'] = platform.version()
+        system_environment['python_version'] = sys.version
+
         cle.extra_data['spdx_license_list_version'] = scancode_config.spdx_license_list_version
         if outdated:
             cle.extra_data['OUTDATED'] = outdated
@@ -881,7 +899,7 @@ def run_scan(
         success = success and pre_scan_success
 
         ########################################################################
-        # 6. run scans.
+        # Run scans
         ########################################################################
 
         scan_success = run_scanners(
@@ -899,7 +917,7 @@ def run_scan(
         success = success and scan_success
 
         ########################################################################
-        # 7. run postscans
+        # Run postscans
         ########################################################################
 
         # TODO: add progress indicator
@@ -917,7 +935,7 @@ def run_scan(
         success = success and post_scan_success
 
         ########################################################################
-        # 8. apply output filters
+        # Apply output filters
         ########################################################################
 
         # TODO: add progress indicator
@@ -935,7 +953,7 @@ def run_scan(
         success = success and output_filter_success
 
         ########################################################################
-        # 9. save outputs
+        # Save outputs
         ########################################################################
 
         counts = codebase.compute_counts(skip_root=strip_root, skip_filtered=True)
@@ -969,7 +987,7 @@ def run_scan(
             success = success and output_success
 
         ########################################################################
-        # 9. display summary
+        # Display summary
         ########################################################################
         codebase.timings['total'] = time() - processing_start
 
@@ -986,12 +1004,14 @@ def run_scan(
             )
 
         ########################################################################
-        # 10. optionally assemble results to return
+        # Optionally assemble results to return
         ########################################################################
         if return_results:
             # the structure is exactly the same as the JSON output
             from formattedcode.output_json import get_results
             results = get_results(codebase, as_list=True, **requested_options)
+        elif return_codebase:
+            results = codebase
 
     finally:
         # remove temporary files
@@ -1114,8 +1134,7 @@ def run_scanners(
 
     progress_manager = None
     if not quiet:
-        echo_func('Scan files for: %(scan_names)s '
-                    'with %(processes)d process(es)...' % locals())
+        echo_func(f'Scan files for: {scan_names} with {processes} process(es)...')
         item_show_func = partial(path_progress_message, verbose=verbose)
         progress_manager = partial(progressmanager,
             item_show_func=item_show_func,
@@ -1177,16 +1196,13 @@ def scan_codebase(
     execution time (as a float in seconds). This is added to the `scan_timings`
     mapping of each Resource as {scanner.name: execution time}.
 
-    Provide optional progress feedback in the UI using the `progress_manager`
-    callable that accepts an iterable of tuple of (location, rid, scan_errors,
-    scan_result ) as argument.
+    Provide optional progress feedback in the UI using the ``progress_manager``
+    callable that accepts an iterable of tuple of (location, path, scan_errors,
+    scan_result) as argument.
     """
 
-    # FIXME: this path computation is super inefficient tuples of  (absolute
-    # location, resource id)
-
     # NOTE: we never scan directories
-    resources = ((r.location, r.rid) for r in codebase.walk() if r.is_file)
+    resources = ((r.location, r.path) for r in codebase.walk() if r.is_file)
 
     use_threading = processes >= 0
     runner = partial(
@@ -1229,7 +1245,7 @@ def scan_codebase(
         while True:
             try:
                 (location,
-                 rid,
+                 path,
                  scan_errors,
                  scan_time,
                  scan_result,
@@ -1239,14 +1255,14 @@ def scan_codebase(
                     logger_debug(
                     'scan_codebase: location:', location, 'results:', scan_result)
 
-                resource = get_resource(rid)
+                resource = get_resource(path=path)
 
                 if not resource:
                     # this should never happen
                     msg = (
                         'ERROR: Internal error in scan_codebase: Resource '
-                        'at %(rid)r is missing from codebase.\n'
-                        'Scan result not saved:\n%(scan_result)r.' % locals()
+                        f'at {path!r} is missing from codebase.\n'
+                        f'Scan result not saved:\n{scan_result!r}.'
                     )
                     codebase.errors.append(msg)
                     success = False
@@ -1319,22 +1335,22 @@ def terminate_pool_with_backoff(pool, number_of_trials=3):
 
 
 def scan_resource(
-    location_rid,
+    location_path,
     scanners,
     timeout=DEFAULT_TIMEOUT,
     with_timing=False,
     with_threading=True,
 ):
     """
-    Return a tuple of:
-        (location, rid, scan_errors, scan_time, scan_results, timings)
-    by running the `scanners` Scanner objects for the file or directory resource
-    with id `rid` at `location` provided as a `location_rid` tuple of (location,
-    rid) for up to `timeout` seconds. If `with_threading` is False, threading is
-    disabled.
+    Given a ``location_path`` tuple pf (location, path), return a tuple of:
+        (location, path, scan_errors, scan_time, scan_results, timings)
+    by running the ``scanners`` Scanner objects for the file or directory
+    resource at ``location`` and ``path`` for up to ``timeout`` seconds. If
+    ``with_threading`` is False, threading is disabled. Include detailed timings
+    if ``with_timing`` is True.
 
     The returned tuple has these values:
-    - `location` and `rid` are the original arguments.
+    - `location` and `path` are the original arguments.
     - `scan_errors` is a list of error strings.
     - `scan_results` is a mapping of scan results from all scanners.
     - `scan_time` is the duration in seconds to run all scans for this resource.
@@ -1342,11 +1358,11 @@ def scan_resource(
       tracking the execution duration each each scan individually.
       `timings` is empty unless `with_timing` is True.
 
-    All these values MUST be serializable/pickable because of the way multi-
-    processing/threading works.
+    All these values MUST be serializable and pickable because of the way multi-
+    processing and threading works.
     """
     scan_time = time()
-    location, rid = location_rid
+    location, path = location_path
     results = {}
     scan_errors = []
     timings = {} if with_timing else None
@@ -1372,7 +1388,7 @@ def scan_resource(
             else:
                 deadline = sys.maxsize
 
-            runner = partial(scanner.function, location, deadline=deadline)
+            runner = partial(scanner.function, location, path=path, deadline=deadline)
             error, values_mapping = interruptor(runner, timeout=timeout)
             if error:
                 msg = 'ERROR: for scanner: ' + scanner.name + ':\n' + error
@@ -1390,7 +1406,7 @@ def scan_resource(
 
     scan_time = time() - scan_time
 
-    return location, rid, scan_errors, scan_time, results, timings
+    return location, path, scan_errors, scan_time, results, timings
 
 
 def display_summary(codebase, scan_names, processes, errors, echo_func=echo_stderr):
@@ -1651,3 +1667,9 @@ def get_pretty_params(ctx, generic_paths=False):
             options.append((cli_opt, value))
 
     return dict(sorted(args) + sorted(options))
+
+
+if __name__ == '__main__':
+    # We have this __main__ block so that we can run scancode as a script.
+    # This is needed so we can use the Python debugging features in VSCode.
+    scancode()
