@@ -10,12 +10,12 @@
 import datetime
 import errno
 import os
+import tempfile
 from os.path import abspath
 from os.path import dirname
+from os.path import exists
 from os.path import expanduser
 from os.path import join
-from os.path import exists
-import tempfile
 
 """
 Core configuration globals.
@@ -77,17 +77,17 @@ def _create_dir(location):
 
 # in case package is not installed or we do not have setutools/pkg_resources
 # on hand fall back to this version
-__version__ = '30.1.0'
+__version__ = '31.0.2'
 
 # used to warn user when the version is out of date
-__release_date__ = datetime.datetime(2021, 9, 24)
+__release_date__ = datetime.datetime(2022, 8, 17)
 
 # See https://github.com/nexB/scancode-toolkit/issues/2653 for more information
 # on the data format version
-__output_format_version__ = '1.0.0'
+__output_format_version__ = '2.0.0'
 
 #
-spdx_license_list_version = '3.14'
+spdx_license_list_version = '3.17'
 
 try:
     from pkg_resources import get_distribution, DistributionNotFound
@@ -106,11 +106,7 @@ scancode_root_dir = dirname(scancode_src_dir)
 # USAGE MODE FLAGS
 ################################################################################
 
-# tag file or env var to determined if we are in dev mode
-SCANCODE_DEV_MODE = (
-    os.getenv('SCANCODE_DEV_MODE')
-    or exists(join(scancode_root_dir, 'SCANCODE_DEV_MODE'))
-)
+_SCANCODE_DEV_MODE = os.path.exists(join(scancode_root_dir, '.git'))
 
 ################################################################################
 # USAGE MODE-, INSTALLATION- and IMPORT- and RUN-SPECIFIC DIRECTORIES
@@ -128,8 +124,8 @@ available across runs from the value of the `SCANCODE_CACHE` environment
 variable. If `SCANCODE_CACHE` is not set, a default sub-directory in the user
 home directory is used instead.
 """
-if SCANCODE_DEV_MODE:
-    # in dev mode the cache and temp files are stored execlusively under the
+if _SCANCODE_DEV_MODE:
+    # in dev mode the cache and temp files are stored exclusively under the
     # scancode_root_dir
     scancode_cache_dir = join(scancode_root_dir, '.cache')
 else:
@@ -166,7 +162,7 @@ is used instead. Each scan run creates its own tempfile subdirectory.
 __scancode_temp_base_dir = os.getenv('SCANCODE_TEMP')
 
 if not __scancode_temp_base_dir:
-    if SCANCODE_DEV_MODE:
+    if _SCANCODE_DEV_MODE:
         __scancode_temp_base_dir = join(scancode_root_dir, 'tmp')
     else:
         __scancode_temp_base_dir = system_temp_dir
@@ -174,3 +170,7 @@ if not __scancode_temp_base_dir:
 _create_dir(__scancode_temp_base_dir)
 _prefix = 'scancode-tk-' + __version__ + '-'
 scancode_temp_dir = tempfile.mkdtemp(prefix=_prefix, dir=__scancode_temp_base_dir)
+
+
+# Used for tests to regenerate fixtures with regen=True
+REGEN_TEST_FIXTURES = os.getenv('SCANCODE_REGEN_TEST_FIXTURES', False)
